/*
 * Copyright 2012-16 Fraunhofer ISE
 *
 * This file is part of jDLMS.
 * For more information visit http://www.openmuc.org
 *
 * jDLMS is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * jDLMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with jDLMS.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package org.openmuc.jdlms.internal;

public enum AuthenticationMechanism {

	/**
	 * No authentication used.
	 */
	NONE(0),

	/**
	 * Authentication of the client by sending a shared password as secret
	 */
	LOW(1),

	/**
	 * Authentication of both client and smart meter by manufacturer specific method
	 */
	HLS2_MANUFACTORER(2),

	/**
	 * Authentication of both client and smart meter using MD5 and a pre shared secret password
	 */
	HLS3_MD5(3),

	/**
	 * Authentication of both client and smart meter using SHA-1 and a pre shared secret password
	 */
	HLS4_SHA1(4),

	/**
	 * Authentication of both client and smart meter using GMAC and a pre shared secret password
	 */
	HLS5_GMAC(5),

	/**
	 * Authentication of both client and smart meter using SHA-256 and a pre shared secret password
	 */
	HLS6_SHA256(6),

	/**
	 * Authentication of both client and smart meter using ECDSA and a pre shared secret password
	 */
	HLS7_ECDSA(7);

	private final int code;

	private static final AuthenticationMechanism[] values = AuthenticationMechanism.values();

	private AuthenticationMechanism(int code) {
		this.code = code;
	}

	public byte getCode() {
		return (byte) code;
	}

	public static AuthenticationMechanism newAuthenticationMechanism(int code) {
		for (AuthenticationMechanism interfaceClass : values) {
			if (interfaceClass.code == code) {
				return interfaceClass;
			}
		}
		throw new IllegalArgumentException("Unknown Flag code: " + code);
	}

}
